
#include "Arduino.h"
#define MVT_PERIODIQUE 100

int DIR = 4;  // PG5 (portG sortie 5) sur MEGA, PC4 sur UNO
int STEP = 5; // PE3 (port E sortie 3) sur MEGA, PC5 sur UNO


uint16_t N = 8;
uint8_t P = 4;
uint16_t delais[32] = {1,2,4,4}; // délais entre les micropas

int8_t sens_rotation[4] = {1,-1,-1,1};
uint16_t indice_delai;
int8_t increment_indice_delai;
uint16_t compteur_delai;
uint16_t compteur_n;
uint16_t compteur_p;
uint8_t compteur_quart_periode;
uint16_t compteur_periode;
uint16_t nombre_periodes;
uint8_t flip;

uint16_t diviseur[6] = {0, 1, 8, 64, 256, 1024};
             
void timer1_init(uint32_t period) {
  TCCR1A  = 0;
  TCCR1B = (1 << WGM12); // mode CTC avec OCR1A pour le maximum
  uint32_t top = (F_CPU / 1000000 * period);
  int clock = 1;
  while ((top > 0xFFFF) && (clock < 5)) {
    clock++;
    top = (F_CPU / 1000000 * period / diviseur[clock]);
  }
  OCR1A = top;
  TIMSK1 = (1 << OCIE1A);
  TCCR1B |= clock;
}
             
ISR(TIMER1_COMPA_vect) {
  char step = 0;
  char init_delai=0;
  if (flip==0) {
      flip=1;
      compteur_delai--;
      if (compteur_delai==0) step=1;
      compteur_n++;
      if (compteur_n==N) {
          compteur_n=0;
          compteur_p++;
          if (compteur_p==P) {
              indice_delai += increment_indice_delai;
              compteur_p = 0;
              increment_indice_delai = -increment_indice_delai;
              compteur_quart_periode++;
              if ((compteur_quart_periode==1)||(compteur_quart_periode==3)) {
                step=0;
                init_delai = 1;
              }
              if (compteur_quart_periode == 4) {
                  compteur_quart_periode = 0;
                  compteur_periode++;
                  if (compteur_periode == nombre_periodes) {
                      TCCR1B = 0; // fin du mouvement
                    // D5 = LOW
                    #if defined(__AVR_ATmega2560__)
                      PORTE &= ~(1 << PORTE3);
                    #elif defined(__AVR_ATmega32U4__)
                      PORTC &= ~(1 << PORTC6);
                    #else
                      PORTC &= ~(1 << PORTC5);
                    #endif
                      Serial.write(1);
                  }
              }
              if (sens_rotation[compteur_quart_periode]==1) {
                    // D4 = HIGH
                    #if defined(__AVR_ATmega2560__)
                        PORTG |= (1 << PORTG5); 
                    #elif defined(__AVR_ATmega32U4__)
                        PORTD |= (1 << PORTD4);
                    #else
                        PORTC |= (1 << PORTC4);
                    #endif
              }
              else {
                    // D4 = LOW
                    #if defined(__AVR_ATmega2560__)
                        PORTG &= ~(1 << PORTG5); 
                    #elif defined(__AVR_ATmega32U4__)
                        PORTD &= ~(1 << PORTD4);
                    #else
                        PORTC &= ~(1 << PORTC4);
                    #endif 
              }
          }
          indice_delai += increment_indice_delai;
      }
      if (step) {
             step=0;
             // D5 = HIGH
             #if defined(__AVR_ATmega2560__)
                PORTE |= (1 << PORTE3);
             #elif defined(__AVR_ATmega32U4__)
                PORTC |= (1 << PORTC6);
             #else
                PORTC |= (1 << PORTC5);
             #endif   
             compteur_delai = delais[indice_delai];
      }
      if (init_delai) {
          init_delai=0;
          compteur_delai = delais[indice_delai];
      }
  }
  else {
      // D5 = LOW
      #if defined(__AVR_ATmega2560__)
              PORTE &= ~(1 << PORTE3);
      #elif defined(__AVR_ATmega32U4__)
              PORTC &= ~(1 << PORTC6);
      #else
              PORTC &= ~(1 << PORTC5);
      #endif
      flip=0;
  }  
}
               
void setup() {
    char c;
    Serial.begin(115200);
    Serial.setTimeout(0);
    c = 0;
    Serial.write(c);
    c = 255;
    Serial.write(c);
    c = 0;
    Serial.write(c);
    pinMode(DIR,OUTPUT);
    pinMode(STEP,OUTPUT);
    digitalWrite(DIR,HIGH);
    digitalWrite(STEP,LOW);
    
    compteur_n = 0;
    compteur_p = 0;
    compteur_delai = 0;
    indice_delai = 0;
    compteur_delai = delais[indice_delai];
    increment_indice_delai = 1;
    compteur_periode = 0;
    flip=0;
    
    nombre_periodes = 100;
    float T = 1000.0;
    float periode_echant = T/(4*N*P);
    timer1_init(periode_echant*1e3*0.5);
}
                
void lecture_mvt_periodique() {
    uint32_t c1,c2,c3,c4;
    while (Serial.available()<2) {};
    c1 = Serial.read();
    c2 = Serial.read();
    N = ((c1<<8) | c2);
    while (Serial.available()<1) {};
    P = Serial.read();
    for (int p=0; p<P; p++) {
        while (Serial.available()<2) {};
        c1 = Serial.read();
        c2 = Serial.read();
        delais[p] = ((c1<<8) | c2);
    }
    while (Serial.available()<2) {};
    c1 = Serial.read();
    c2 = Serial.read();
    uint16_t T = ((c1<<8) | c2); // période en millisecondes
    while (Serial.available()<2) {};
    c1 = Serial.read();
    c2 = Serial.read();
    nombre_periodes = ((c1<<8) | c2);
    digitalWrite(DIR,HIGH);
    digitalWrite(STEP,LOW);
    compteur_n = 0;
    compteur_p = 0;
    compteur_delai = 0;
    indice_delai = 0;
    compteur_delai = delais[indice_delai];
    increment_indice_delai = 1;
    compteur_periode = 0;
    float periode_echant = ((float)T)/(4*N*P);
    flip=0;
    timer1_init(periode_echant*1000*0.5);
}                 
                 
void lecture_serie() {
  char com;
  if (Serial.available()>0) {
    com = Serial.read();
    if (com==MVT_PERIODIQUE) lecture_mvt_periodique();
  }  
}
                 
void loop() {
   delay(100);
   lecture_serie();
}
                 